﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Drawing.Drawing2D;
using System.IO;
using System.Drawing.Imaging;

namespace howto_scribble_on_bg
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // The polylines we draw.
        private List<List<Point>> Polylines = new List<List<Point>>();

        // The new polyline we are drawing.
        private List<Point> NewPolyline = null;

        // Start drawing.
        private void picCanvas_MouseDown(object sender, MouseEventArgs e)
        {
            // Create the new polyline.
            NewPolyline = new List<Point>();
            Polylines.Add(NewPolyline);

            // Add the first point.
            NewPolyline.Add(e.Location);
        }

        // Continue drawing.
        private void picCanvas_MouseMove(object sender, MouseEventArgs e)
        {
            if (NewPolyline == null) return;
            NewPolyline.Add(e.Location);
            picCanvas.Refresh();
        }

        // Stop drawing.
        private void picCanvas_MouseUp(object sender, MouseEventArgs e)
        {
            if (NewPolyline == null) return;

            // See of the new polyline contains more than 1 point.
            if (NewPolyline.Count < 2)
            {
                // Remove it.
                Polylines.RemoveAt(Polylines.Count - 1);
            }

            NewPolyline = null;
            picCanvas.Refresh();
        }

        // Redraw.
        private void picCanvas_Paint(object sender, PaintEventArgs e)
        {
            e.Graphics.SmoothingMode = SmoothingMode.AntiAlias;

            // Draw the polylines.
            foreach (List<Point> polyline in Polylines)
            {
                e.Graphics.DrawLines(Pens.Black, polyline.ToArray());
            }
        }

        private void mnuFileExit_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        // Start a new drawing.
        private void mnuFileNew_Click(object sender, EventArgs e)
        {
            Polylines = new List<List<Point>>();
            picCanvas.Image = null;

            int margin = picCanvas.Left;
            int width = ClientSize.Width - 2 * margin;
            int height = ClientSize.Height - picCanvas.Top - margin;
            picCanvas.SizeMode = PictureBoxSizeMode.Normal;
            picCanvas.Size = new Size(width, height);
            picCanvas.Anchor =
                AnchorStyles.Top |
                AnchorStyles.Bottom |
                AnchorStyles.Left |
                AnchorStyles.Right;

            picCanvas.Refresh();
        }

        private void mnuFileSave_Click(object sender, EventArgs e)
        {
            if (sfdResult.ShowDialog() == DialogResult.OK)
            {
                Bitmap bm = GetControlImage(picCanvas);
                SaveImage(bm, sfdResult.FileName);
            }
        }

        // Return a Bitmap holding an image of the control.
        private Bitmap GetControlImage(Control ctl)
        {
            Bitmap bm = new Bitmap(ctl.Width, ctl.Height);
            ctl.DrawToBitmap(bm,
                new Rectangle(0, 0, ctl.Width, ctl.Height));
            return bm;
        }

        // Save the file with the appropriate format.
        public void SaveImage(Image image, string filename)
        {
            string extension = Path.GetExtension(filename);
            switch (extension.ToLower())
            {
                case ".bmp":
                    image.Save(filename, ImageFormat.Bmp);
                    break;
                case ".exif":
                    image.Save(filename, ImageFormat.Exif);
                    break;
                case ".gif":
                    image.Save(filename, ImageFormat.Gif);
                    break;
                case ".jpg":
                case ".jpeg":
                    image.Save(filename, ImageFormat.Jpeg);
                    break;
                case ".png":
                    image.Save(filename, ImageFormat.Png);
                    break;
                case ".tif":
                case ".tiff":
                    image.Save(filename, ImageFormat.Tiff);
                    break;
                default:
                    throw new NotSupportedException(
                        "Unknown file extension " + extension);
            }
        }

        private void mnuFileLoadBgImage_Click(object sender, EventArgs e)
        {
            if (ofdBackground.ShowDialog() == DialogResult.OK)
            {
                picCanvas.SizeMode = PictureBoxSizeMode.AutoSize;
                picCanvas.Anchor =
                    AnchorStyles.Top |
                    AnchorStyles.Left;

                picCanvas.Image = LoadBitmapUnlocked(ofdBackground.FileName);
            }
        }

        // Load a bitmap without locking it.
        private Bitmap LoadBitmapUnlocked(string file_name)
        {
            using (Bitmap bm = new Bitmap(file_name))
            {
                return new Bitmap(bm);
            }
        }
    }
}
